#
# Copyright (c) 2012, IDM
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted
# provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright notice, this list of
#      conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list
#      of conditions and the following disclaimer in the documentation and/or other materials
#      provided with the distribution.
#    * Neither the name of the IDM nor the names of its contributors may be used to endorse or
#      promote products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Author: Arnaud de Bossoreille
#
package IDM::SkPublish::API;

use strict;

use LWP;
use URI::Encode qw(uri_encode);

=head1 NAME

<IDM::SkPublish::API> - This the Perl implementation of SkPublish API.

=head1 VERSION

Version 1.0.0

=head1 DESCRIPTION

B<IDM::SkPublish::API> - This the Perl implementation of SkPublish API.

=cut

our $VERSION = '1.0.0';

sub new {
    my $class = shift;
    my $this = bless { }, $class;
    my $baseUrl = shift;
    my $accessKey = shift;
    my $userAgent = shift;
    $this->setBaseUrl($baseUrl);
    $this->setAccessKey($accessKey);
    if(!defined($userAgent)) {
        $userAgent = new LWP::UserAgent;
    }
    $this->setUserAgent($userAgent);
    return $this;
}

sub getAccessKey {
    my $this = shift;
    return $this->{accessKey};
}

sub getBaseUrl {
    my $this = shift;
    return $this->{baseUrl};
}

sub getDictionaries {
    my ($this) = @_;
    my $request = $this->prepareGetRequest($this->{baseUrl}.'dictionaries');
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getDictionary {
    my ($this, $dictionaryCode) = @_;
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    my $request = $this->prepareGetRequest($this->{baseUrl}.'dictionaries/'.$dictionaryCode);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getEntry {
    my ($this, $dictionaryCode, $entryId, $format) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/entries/';
    $uri .= uri_encode($entryId, 1);
    my $c = '?';
    if(defined($format)) {
        if(!$this->isValidEntryFormat($format)) {
            return undef;
        }
        $uri .= $c.'format='.$format;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getEntryPronunciations {
    my ($this, $dictionaryCode, $entryId, $lang) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/entries/';
    $uri .= uri_encode($entryId, 1);
    $uri .= '/pronunciations';
    my $c = '?';
    if(defined($lang)) {
        if(!$this->isValidEntryLang($lang)) {
            return undef;
        }
        $uri .= $c.'lang='.$lang;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getNearbyEntries {
    my ($this, $dictionaryCode, $entryId, $entryNumber) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/entries/';
    $uri .= uri_encode($entryId, 1);
    $uri .= '/nearbyentries';
    my $c = '?';
    if(defined($entryNumber)) {
        $uri .= $c.'entrynumber='.$entryNumber;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getRelatedEntries {
    my ($this, $dictionaryCode, $entryId) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/entries/';
    $uri .= uri_encode($entryId, 1);
    $uri .= '/relatedentries';
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getRequestHandler {
    my $this = shift;
    return $this->{requestHandler};
}

sub getThesaurusList {
    my ($this, $dictionaryCode, $topicId) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/topics/';
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getTopic {
    my ($this, $dictionaryCode, $thesName, $topicId) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/topics/';
    $uri .= uri_encode($thesName, 1);
    $uri .= '/';
    $uri .= uri_encode($topicId, 1);
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getUserAgent {
    my $this = shift;
    return $this->{userAgent};
}

sub getWordOfTheDay {
    my ($this, $dictionaryCode, $day, $format) = @_;
    my $uri = $this->{baseUrl};
    if(defined($dictionaryCode)) {
        if(!$this->isValidDictionaryCode($dictionaryCode)) {
            return undef;
        }
        $uri .= 'dictionaries/'.$dictionaryCode.'/';
    }
    $uri .= 'wordoftheday';
    my $c = '?';
    if(defined($day)) {
        if(!$this->isValidWotdDay($day)) {
            return undef;
        }
        $uri .= $c.'day='.$day;
        $c = '&';
    }
    if(defined($format)) {
        if(!$this->isValidEntryFormat($format)) {
            return undef;
        }
        $uri .= $c.'format='.$format;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub getWordOfTheDayPreview {
    my ($this, $dictionaryCode, $day) = @_;
    my $uri = $this->{baseUrl};
    if(defined($dictionaryCode)) {
        if(!$this->isValidDictionaryCode($dictionaryCode)) {
            return undef;
        }
        $uri .= 'dictionaries/'.$dictionaryCode.'/';
    }
    $uri .= 'wordoftheday/preview';
    my $c = '?';
    if(defined($day)) {
        if(!$this->isValidWotdDay($day)) {
            return undef;
        }
        $uri .= $c.'day='.$day;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub isValidDictionaryCode {
    my ($this, $code) = @_;
    if(length($code) < 1) {
        return 0;
    }
    for(my $i = 0; $i < length($code); ++$i) {
        my $c = substr($code, $i, 1);
        # Make sure no param are injected
        if($c eq '/' || $c eq '%') {
            return 0;
        }
        if($c eq '*' || $c eq '$') {
            return 0;
        }
    }
    return 1;
}

sub isValidEntryFormat {
    my ($this, $format) = @_;
    for(my $i = 0; $i < length($format); ++$i) {
        my $c = substr($format, $i, 1);
        # Make sure no param are injected
        if($c eq '/' || $c eq '%') {
            return 0;
        }
    }
    return 1;
}

sub isValidEntryLang {
    my ($this, $lang) = @_;
    for(my $i = 0; $i < length($lang); ++$i) {
        my $c = substr($lang, $i, 1);
        # Make sure no param are injected
        if($c eq '/' || $c eq '%') {
            return 0;
        }
    }
    return 1;
}

sub isValidWotdDay {
    my ($this, $day) = @_;
    for(my $i = 0; $i < length($day); ++$i) {
        my $c = substr($day, $i, 1);
        # Make sure no param are injected
        if($c eq '/' || $c eq '%') {
            return 0;
        }
    }
    return 1;
}

sub prepareGetRequest {
    my ($this, $uri) = @_;
    my $request = new HTTP::Request(GET => $uri);
    $request->header(accessKey => $this->{accessKey});
    my $rh = $this->{requestHandler};
    if($rh) {
        $rh->prepareGetRequest($request);
    }
    return $request;
}

sub search {
    my ($this, $dictionaryCode, $searchWord, $pageSize, $pageIndex) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/search?q=';
    $uri .= uri_encode($searchWord, 1);
    my $c = '&';
    if(defined($pageSize)) {
        $uri .= $c.'pagesize='.$pageSize;
        $c = '&';
    }
    if(defined($pageIndex)) {
        $uri .= $c.'pageindex='.$pageIndex;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub searchFirst {
    my ($this, $dictionaryCode, $searchWord, $format) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/search/first?q=';
    $uri .= uri_encode($searchWord, 1);
    my $c = '&';
    if(defined($format)) {
        if(!$this->isValidEntryFormat($format)) {
            return undef;
        }
        $uri .= $c.'format='.$format;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub didYouMean {
    my ($this, $dictionaryCode, $searchWord, $entryNumber) = @_;
    my $uri = $this->{baseUrl};
    if(!$this->isValidDictionaryCode($dictionaryCode)) {
        return undef;
    }
    $uri .= 'dictionaries/'.$dictionaryCode.'/search/didyoumean?q=';
    $uri .= uri_encode($searchWord, 1);
    my $c = '&';
    if(defined($entryNumber)) {
        $uri .= $c.'entrynumber='.$entryNumber;
        $c = '&';
    }
    my $request = $this->prepareGetRequest($uri);
    my $ua = $this->{userAgent};
    my $response = $ua->request($request);
    return $response->content;
}

sub setAccessKey {
    my ($this, $accessKey) = @_;
    $this->{accessKey} = $accessKey;
}

sub setBaseUrl {
    my ($this, $baseUrl) = @_;
    if(substr($baseUrl, -1) eq '/') {
        $this->{baseUrl} = $baseUrl;
    } else {
        $this->{baseUrl} = $baseUrl.'/';
    }
}

sub setRequestHandler {
    my ($this, $requestHandler) = @_;
    $this->{requestHandler} = $requestHandler;
}

sub setUserAgent {
    my ($this, $userAgent) = @_;
    $this->{userAgent} = $userAgent;
}

1;

